/*
 * Copyright(c)  2002  Rijkswaterstaat
 *                     Adviesdienst Verkeer en Vervoer
 *                     WSRA
 * File: CRAPTAB.C
 *
 * Nummer kruispunt:        1
 * Regelprogramma vri:      Voorbeeldregeling Fiets- en voetgangersoversteek 
 *
 * Datum:                   20 januari 1996
 * Gewijzigd:               14 maart 2003 
 *                          - aanpassing aan Flash
 *                          - aanpassing HFRVG.INC
 *                          8 september 2003
 *                          - aanpassing aan 1.07.14
 * versie:                  3.1
 */

#include "crtypes.h"
#include "crapcod.h"
#include "craptab.h"
#include "ccie_int.h"
#include "crapdef.h"

#include "voetgnlp.inc"    /* standaard: voetgangersnaloop */

/* 
 * ================================================================
 * declaraties t.b.v. het blokkenschema
 * ================================================================
 */

/*     BLOKKEN schema                                           */
struct BLOKKEN  BLOKKEN_tabel[] = {
/*     blok:    sg:     toewijzing:                             */
/*     BKl,     SGn,    PRIM + WSGR + ALT,                      */
       BK1,     SG07,                 ALT,
       BK1,     SG08,   PRIM,
       BK1,     SG24,   PRIM +        ALT,
       BK1,     SG33,   PRIM +        ALT,
       BK1,     SG34,   PRIM +        ALT,

       BK2,     SG02,   PRIM,
       BK2,     SG03,   PRIM,

       BK3,     SG04,   PRIM,
       BK3,     SG06,   PRIM,
       BK3,     SG07,   PRIM,
       STOP};

/*     Wachtblok                                                */
struct WB  WB_tabel[] = {
/*     wachtblok:                                               */
/*     BKl,                                                     */
       STOP};

/* 
 * ================================================================
 * matrices
 * ================================================================
 */

/*     CONFLICT- en (garantie)ontruimingstijden matrix          */
struct CONFLICT  CONFLICT_tabel[] = {
/*                                    instellingen              */
/*                                    van de tijden:            */
/*     sg:       conflicterende sg:   GO_TIJD:  O_TIJD:         */
/*     SGn,      SGj,                 x.y,      x.y,            */
       SG02,     SG06,                0.0,      0.0,

       SG03,     SG06,                0.0,      0.0,
       SG03,     SG07,                1.0,      2.0,
       SG03,     SG08,                0.0,      1.0,
       SG03,     SG24,                3.0,      4.0,
       SG03,     SG34,                3.0,      4.0,

       SG04,     SG08,                0.0,      0.0,
       SG04,     SG24,                0.0,      0.0,
       SG04,     SG33,                0.0,      1.0,

       SG06,     SG02,                1.0,      2.0,
       SG06,     SG03,                1.0,      2.0,
       SG06,     SG08,                0.0,      0.0,
       SG06,     SG24,                0.0,      0.0,
       SG06,     SG33,                0.0,      1.0,

       SG07,     SG03,                0.0,      0.0,
       SG07,     SG24,                2.0,      2.0,
       SG07,     SG34,                2.0,      3.0,

       SG08,     SG03,                1.0,      1.0,
       SG08,     SG04,                1.0,      2.0,
       SG08,     SG06,                1.0,      1.0,

       SG24,     SG03,                0.0,      0.0,
       SG24,     SG04,                4.0,      5.0,
       SG24,     SG06,                3.0,      4.0,
       SG24,     SG07,                0.0,      0.0,

       SG33,     SG04,                6.0,      6.0,
       SG33,     SG06,                6.0,      6.0,

       SG34,     SG03,                0.0,      1.0,
       SG34,     SG07,                2.0,      3.0,
       STOP};

/*     FICTief CONFLICT matrix                                  */
struct FICT_CONFLICT  FICT_CONFLICT_tabel[] = {
/*     sg:            fictief conflicterende sg:                */
/*     SGn,           SGj,                                      */
       SG03,          SG33,
       SG04,          SG34,
       SG06,          SG34,
       SG07,          SG33,
       STOP};

/*     extra op te geven SG'en k volgens BS_H5 formule t        */
struct SG_BS_H5t  SG_BS_H5t_tabel[] = {
/*     sg:        sg k volgens BS_H5 formule t:                 */
/*     SGn,       SGk,                                          */
       STOP};

/*     extra op te geven SG'en j volgens BS_H5 formule ac       */
struct SG_BS_H5ac  SG_BS_H5ac_tabel[] = {
/*     sg:        sg k volgens BS_H5 formule ac:                */
/*     SGn,       SGk,                                          */
       SG03,      SG33,
       SG04,      SG34,
       SG06,      SG34,
       SG07,      SG33,
       STOP};

/*     gekoppelde signaalgroepen volgens BS_H6.6                */
struct SG_BS_H66  SG_BS_H66_tabel[] = {
/*     sg:        gekoppelde sg:                                */
/*     SGn,       SGj,                                          */
       SG33,      SG34,
       STOP};

/* 
 * ================================================================
 * declaraties t.b.v. de signaalgroepprocedure 
 * ================================================================
 */

/*     SignaalGroep_TIJDEN tabel                                */
struct SG_TIJDEN  SG_TIJDEN_tabel[] = {
/*            instellingen van de tijden:                       */
/*     sg:    GG_TIJD:  VG_TIJD:  GGL_TIJD:  GL_TIJD:  GR_TIJD: */
/*     SGn,    x.y,      x.y,      x.y,       x.y,      x.y,    */
       SG02,   6.0,      6.0,      3.0,       4.0,      2.0,
       SG03,   6.0,      6.0,      3.0,       3.0,      2.0,
       SG04,   6.0,      6.0,      3.0,       3.0,      2.0,
       SG06,   6.0,      6.0,      3.0,       3.0,      2.0,
       SG07,   6.0,      6.0,      3.0,       3.0,      2.0,
       SG08,   6.0,      6.0,      3.0,       4.0,      2.0,
       SG24,   5.0,      5.0,      2.0,       2.0,      2.0,
       SG33,   5.0,      5.0,      3.0,       3.0,      2.0,
       SG34,   5.0,      5.0,      3.0,       3.0,      2.0,
       STOP};

/*     MG_TIJDEN tabel                                          */
struct MG_TIJDEN  MG_TIJDEN_tabel[] = {
/*            instellingen (x.y of NG) van de tijden:           */
/*     sg:    MG_TIJD:  MG_TIJD1:  MG_TIJD2:  MG_TIJD3:         */
/*     SGn,    x.y,      x.y,       x.y,       x.y,             */
       SG02,    NG,     25.0,        NG,        NG,
       SG03,    NG,     15.0,        NG,        NG,
       SG04,    NG,     20.0,        NG,        NG,
       SG06,    NG,     20.0,        NG,        NG,
       SG07,    NG,     15.0,        NG,        NG,
       SG08,    NG,     30.0,        NG,        NG,
       SG24,    NG,     15.0,        NG,        NG,
       STOP};

/*     Extra MG_TIJDEN tabel                                    */
struct E_MG_TIJDEN  E_MG_TIJDEN_tabel[] = {
/*            instellingen (x.y of NG) van de tijden:           */
/*     sg:    MG_TIJD4:  MG_TIJD5:  MG_TIJD6:                   */
/*     SGn;    x.y,       x.y,       x.y,                       */
       STOP};

/*     HiaatTIJDEN tabel                                        */
struct H_TIJDEN  H_TIJDEN_tabel[] = {
/*               instellingen (x.y of NG) van de tijden:        */
/*     sg:       H1e_TIJD:   H2e_TIJD:                          */
/*     SGn,       x.y,        x.y,                              */
       SG02,      0.1,        NG,       
       SG03,      0.1,        NG,      
       SG04,      0.1,        NG,      
       SG06,      0.1,        NG,      
       SG07,      0.1,        NG,      
       SG08,      0.1,        NG,      
       SG24,      2.5,        NG,
       STOP};

/*     Extra-HiaatTIJDEN tabel                                  */
struct E_H_TIJDEN  E_H_TIJDEN_tabel[] = {
/*               instellingen (x.y of NG) van de tijden:        */
/*     sg:       E_H1e_TIJD:    E_H2e_TIJD:                     */
/*     SGn,       x.y,           x.y,                           */
       STOP};

/*     SignaalGroep-DETECTIE tabel                              */
struct SG_DETECTIE           SG_DETECTIE_tabel[] = {
/*     D():     type:        sg:     toewijzing:                */
/*     SGn_i,   DP (of DK),  SGn,    AVR + BS_H5p + H1e + H2e,  */
       SG02_1,  DP,          SG02,   AVR + BS_H5p,
       SG02_2,  DP,          SG02,   AVR + H1e,
       SG02_3,  DP,          SG02,   AVR + H1e,

       SG03_1,  DP,          SG03,   AVR + BS_H5p,
       SG03_2,  DP,          SG03,   AVR + H1e,

       SG04_1,  DP,          SG04,   AVR + BS_H5p,
       SG04_2,  DP,          SG04,   AVR + H1e,
       SG04_3,  DP,          SG04,   AVR + H1e,

       SG06_1,  DP,          SG06,   AVR + BS_H5p,
       SG06_2,  DP,          SG06,   AVR + H1e,
       SG06_3,  DP,          SG06,   AVR + H1e,

       SG07_1,  DP,          SG07,   AVR + BS_H5p,
       SG07_2,  DP,          SG07,   AVR + H1e,

       SG08_1,  DP,          SG08,   AVR + BS_H5p,
       SG08_2,  DP,          SG08,   AVR + H1e,
       SG08_3,  DP,          SG08,   AVR + H1e,

       SG24_1,  DP,          SG24,   AVR + H1e,
       SG24_2,  DK,          SG24,   AVR,

       SG33_1,  DK,          SG33,   AVR, /* BUITENDRUKKNOP */
       SG33_2,  DK,          SG33,   AVR,

       SG34_1,  DK,          SG34,   AVR,
       SG34_2,  DK,          SG34,   AVR, /* BUITENDRUKKNOP */
       STOP};

/*     tabel voor SG'en met RichtingGevoelige Aanvraag          */
struct SG_RGA     SG_RGA_tabel[] = {
/*     1e D():    2e D():    koplus D():  sg:   paarnr:  TIJD():*/
/*     SGn_i,     SGn_j,     SGn_k,       SGn,       x,  SGn_l, */
       STOP};

/*     FIETS of VOETGangersRichting tabel                       */
struct FIETS_VOETG_R  FIETS_VOETG_R_tabel[] = {
/*     fiets- of voetgangersrichting:                           */
/*     SGn,                                                     */
       SG24,
       SG33,
       SG34,
       STOP};

/* 
 * ================================================================
 * declaraties t.b.v. standaardapplicaties
 * ================================================================
 */

/*     tabel voor SG'en die gebruik maken van hulpfunctie MAH3  */
struct SG_MAH3  SG_MAH3_tabel[] = {
/*     sg die gebruik maakt van de hulpfunctie MAH3:            */
/*     SGn,                                                     */
       STOP};

/*     tabel voor SG'en die gebruik maken van PPAH en VNMH      */
struct SG_PPAH  SG_PPAH_tabel[] = {
/*     PPAH() (en VNMH()):  TIJD():                             */
/*     SGn,                 SGn_i,                              */
       SG08,                T_PPA_08,
       STOP};

/*     tabel voor toepassen WSRH                                */
struct SG_WSRH  SG_WSRH_tabel[] = {
/*     TOEPASSEN,                                               */
       TOEPASSEN,
       STOP};

/* 
 * ================================================================
 * tijden, klokken en schakelaars
 * ================================================================
 */

/*     TIJDENtabel                                              */
struct TIJDEN                      TIJDEN_tabel[] = {
/*     TIJD():                     instelling:                  */
/*     SGn_i,                       x.y,                        */
       T_NALOOP33_34,              12.0,  /* naloop 33 -> 34    */
       T_NALOOP34_33,              12.0,  /* naloop 34 -> 33    */
       T_PPA_08,                   25.0,  /* PPAH             */
       STOP};

/*     KLOKKENtabel                                             */
struct KLOKKEN KLOKKEN_tabel[] = {
/*     KL():                    1e tijdstip:  2e tijdstip:      */
/*     NAAM,                     x.yz,         x.yz,            */
       STOP};

/*     SCHAKELAARtabel                                          */
struct SCHAKELAAR SCHAKELAAR_tabel[] = {
/*     SCH():                   instelling:                     */
/*     NAAM,                    AAN (of UIT),                   */
       STOP};

/*     extra EGGPARM tabel                                      */
struct EGGPARMS  EGGPARMS_tabel[] = {
/*     EGGPARM(),                 instelling                    */
       STOP};

/*     extra PARMS tabel  */
struct EPARMS       EPARMS_tabel[] = {
/*     EPARM(),     instelling    */
       STOP};

/* 
 * ================================================================
 * opties voor het vasthouden van de blokactiviteit
 * ================================================================
 */

/*     BK gebonden optie: BO_OP(BKl)                            */
Bool alg_BO_funct (Int16 n, Int16 i) { 
     return ROG(n) && A(n) || RVG(n) || G(n) && TIJD(i) && !MVG(n); }
struct BK_OPTIE           BO_OP_tabel[] = {
/*     optie voor blok:   inhoud van de optie:                  */
/*     BKl,               functie,                              */
       STOP};

/* 
 * ================================================================
 * opties voor aanvragen, meeverlengen en tegenhouden
 * ================================================================
 */

/*     SG gebonden optie: MA_OP(SGn)                            */
Bool MA_funct_24 (void){ return A(SG33) || A(SG34);}
Bool MA_funct_33 (void){ return A(SG34); }
Bool MA_funct_34 (void){ return A(SG33); }
struct SG_OPTIE           MA_OP_tabel[] = {
/*     optie voor sg:     inhoud van de optie:                  */
/*     SGn,               functie,                              */
       SG24,              MA_funct_24,
       SG33,              MA_funct_33,
       SG34,              MA_funct_34,
       STOP};

/*     SG en BK gebonden optie:    PPP_OP(SGn, BKl)             */
struct SG_BK_OPTIE                 PPP_OP_tabel[] = {
/*     optie voor sg:  voor blok:  inhoud van de optie:         */
/*     SGn,            BKl,        functie,                     */
       STOP};

/*     SG gebonden optie: MVG_OP(SGn)                           */
Bool MVG_funct(void) { return WSRH; }
struct SG_OPTIE           MVG_OP_tabel[] = {
/*     optie voor sg:     inhoud van de optie:                  */
/*     SGn,               WAAR (of functie),                    */
       SG02,              MVG_funct,
       SG03,              MVG_funct,
       SG04,              MVG_funct,
       SG06,              MVG_funct,
       SG07,              MVG_funct,
       SG08,              MVG_funct,
       SG24,              MVG_funct,
       STOP};

/*     SG gebonden optie: TR_OP(SGn)                            */
struct SG_OPTIE           TR_OP_tabel[] = {
/*     optie voor sg:     inhoud van de optie:                  */
/*     SGn,               functie,                              */
       STOP};

/* 
 * ================================================================
 * opties voor versnelde realisatie
 * ================================================================
 */

/*     SG gebonden optie: VAA_OP(SGn)                           */
struct SG_OPTIE           VAA_OP_tabel[] = {
/*     optie voor sg:     inhoud van de optie:                  */
/*     SGn,               WAAR (of functie),                    */
       SG02,              WAAR,
       SG03,              WAAR,
       SG04,              WAAR,
       SG06,              WAAR,
       SG07,              WAAR,
       SG08,              WAAR,
       SG24,              WAAR,
       SG33,              WAAR,
       SG34,              WAAR,
       STOP};

/*     SG gebonden optie: PPV_OP(SGn)                           */
struct SG_OPTIE           PPV_OP_tabel[] = {
/*     optie voor sg:     inhoud van de optie:                  */
/*     SGn,               WAAR (of functie),                    */
       SG02,              WAAR,
       SG03,              WAAR,
       SG04,              WAAR,
       SG06,              WAAR,
       SG07,              WAAR,
       SG08,              WAAR,
       SG24,              WAAR,
       SG33,              WAAR,
       SG34,              WAAR,
       STOP};

/* 
 * ================================================================
 * optie voor bijzondere realisatie
 * ================================================================
 */

/*     SG gebonden optie: PPB_OP(SGn)                           */
struct SG_OPTIE           PPB_OP_tabel[] = {
/*     optie voor sg:     inhoud van de optie:                  */
/*     SGn,               functie,                              */
       STOP};

/* 
 * ================================================================
 * opties voor alternatieve realisatie
 * ================================================================
 */

/*     SG en BK gebonden optie:    PPA_OP(SGn, BKl)             */
Bool PPA_funct_BK1 (void) { return PPAH(SG08); }
struct SG_BK_OPTIE                 PPA_OP_tabel[] = {
/*     optie voor sg:  voor blok:  inhoud van de optie:         */
/*     SGn,            BKl,        functie,                     */
       SG07,           BK1,        PPA_funct_BK1,
       SG24,           BK1,        PPA_funct_BK1,
       SG33,           BK1,        PPA_funct_BK1,
       SG34,           BK1,        PPA_funct_BK1,
       STOP};

/*     SG en BK gebonden optie:    VNM_OP(SGn, BKl)             */
Bool VNM_funct_BK1    (void) { return VNMH(SG08); }
struct SG_BK_OPTIE                 VNM_OP_tabel[] = {
/*     optie voor sg:  voor blok:  inhoud van de optie:         */
/*     SGn,            BKl,        functie,                     */
       SG07,           BK1,        VNM_funct_BK1,
       SG24,           BK1,        VNM_funct_BK1,
       SG33,           BK1,        VNM_funct_BK1,
       SG34,           BK1,        VNM_funct_BK1,
       STOP};

/* 
 * ================================================================
 * opties voor benvloeding van het verloop van de signaalgroep
 * ================================================================
 */

/*     SG gebonden optie: AFK_OP(SGn)                           */
struct SG_OPTIE           AFK_OP_tabel[] = {
/*     optie voor sg:     inhoud van de optie:                  */
/*     SGn,               functie,                              */
       STOP};

/*     SG gebonden optie: BL_OP(SGn)                            */
struct SG_OPTIE           BL_OP_tabel[] = {
/*     optie voor sg:     inhoud van de optie:                  */
/*     SGn,               functie,                              */
       STOP};

/*     SG gebonden optie: RVG_OP(SGn)                           */
Bool RVG_funct_24(void) { return HF(RVG33) || HF(RVG34) || !RVG(SG24); }
Bool RVG_funct_33(void) { return HF(RVG24) || HF(RVG34) || !RVG(SG33); }
Bool RVG_funct_34(void) { return HF(RVG24) || HF(RVG33) || !RVG(SG34); }
struct SG_OPTIE           RVG_OP_tabel[] = {
/*     optie voor sg:     inhoud van de optie:                  */
/*     SGn,               functie,                              */
       SG24,              RVG_funct_24,
       SG33,              RVG_funct_33,
       SG34,              RVG_funct_34,
       STOP};

/*     SG gebonden optie: VAG_OP(SGn)                           */
Bool VAG_funct_33 (void)          
{ return voetgangersnaloop (SG34, SG34_2, DK34, T_NALOOP34_33, BG); }
Bool VAG_funct_34 (void)          
{ return voetgangersnaloop (SG33, SG33_1, DK33, T_NALOOP33_34, EG); }
struct SG_OPTIE           VAG_OP_tabel[] = {
/*     optie voor sg:     inhoud van de optie:                  */
/*     SGn,               functie,                              */
       SG33,              VAG_funct_33,
       SG34,              VAG_funct_34,     
       STOP};

/*     SG gebonden optie: VMG_OP(SGn)                           */
struct SG_OPTIE           VMG_OP_tabel[] = {
/*     optie voor sg:     inhoud van de optie:                  */
/*     SGn,               functie,                              */
       STOP};

/* 
 * ================================================================
 * tabel met koppelsignalen
 * ================================================================
 */
/*     KOPPELtabel                                              */
struct KOPPEL   KOPPEL_tabel[] = {
/*     hulpfunctie:   ingangssignaal:   uitgangssignaal:        */
/*     NAAM,          x (of GEEN),      x (of GEEN),            */
       STOP};

/*     REGELING tabel                                           */
struct REGELING   REGELING_tabel[] = {
/*     Nummer,  */
       1,
       STOP};
